/*
 * $Id: atalk.h,v 1.11 89/03/08 02:58:54 kre Exp $
 *
 *	MEGAN: Multigate Ethernet Gateway to Apple Networks
 *
 *	Code based on SEAGATE by Bill Croft of Stanford.
 *	His copyright still applies.
 *						...kre
 *
 * $Log:	atalk.h,v $
 * Revision 1.11  89/03/08  02:58:54  kre
 * Rename "ddpWKSUnix" to "defddpWKSUnix" since this is now only
 * the default low socket number (before it was THE low socket number)
 * 
 * Revision 1.10  88/09/08  08:37:04  kre
 * Earliest available SUN maintained version (1.10)
 * First genuine distribution version.
 * 
 * Revision 2.1  87/09/27  17:54:17  kre
 * upgrade to multigate V2 .. kip 0287 style
 * Deleted mac only stuff (crud), and added new definitions.
 * 
 * Revision 1.1  86/09/05  20:44:40  kre
 * Initial revision
 */

/*	atalk.h		1.3	02/19/85	*/

/*
 * AppleTalk definitions.
 *
 * Copyright (C) 1984, Apple Computer Inc.
 *  Gene Tyacke, Alan Oppenheimer, G. Sidhu, Rich Andrews.
 *
 * C language version (C) 1984, Stanford Univ. SUMEX project.
 * May be used but not sold without permission.
 */

/*
 * history
 * 08/24/84	GRT	Created.
 * 10/23/84	GRT	
 * 12/01/84	Croft	Created C version;  added packet headers, *Params.
 * 03/16/85	Croft	etc.
 */


/* error codes */

#define	ddpSktErr	-91	/* error in socket number */
#define	ddpLenErr	-92	/* data length too big */
#define	noBridgeErr	-93	/* no net bridge for non-local send */
#define	lapProtErr	-94	/* error in attach/detach */
#define	excessCollsns	-95	/* excessive collisions on write */
#define	portInUse	-97	/* driver Open error */
#define	portNotCf	-98	/* driver Open error */
/* ATP error codes */
#define	reqFailed	-1096	/* SendRequest failed: retrys exceeded */
#define	badATPSkt	-1099	/* bad ATP responding socket */
#define	noRelErr	-1101	/* no release received */
#define	noDataArea	-1104	/* no data area for MPP request */
#define	reqAborted	-1105	/* SendRequest aborted by RelTCB */
/* NBP error codes */
#define	nbpBuffOvr	-1024	/* buffer overflow in LookupName */
#define	nbpNoConfirm	-1025	/* name not confirmed on ConfirmName */
#define	nbpConfDiff	-1026	/* name confirmed at different socket */
#define	nbpDuplicate	-1027	/* duplicate name already exists */
#define	nbpNotFound	-1028	/* name not found on remove */
#define	nbpNISErr	-1029	/* error trying to open the NIS */


/* misc structures */

typedef struct {		/* WDS, write data structure */
	u_short	size;
	u_char	*ptr;
} WDS;

typedef struct {		/* BDS, buffer data structure */
	short	buffSize;
	char	*buffPtr;
	short	dataSize;
	long	userData;
} BDS;

#define	userBytes userData

typedef struct {		/* AddrBlock */
	u_short	net;
	u_char	node;
	u_char	skt;
} AddrBlock;


/*
 * For each protocol, its header structure, definitions, 
 * and control / status parameter block.
 */

struct LAP {			/* LAP */
	u_char	dst;
	u_char	src;
	u_char	type;
};

typedef struct LAP LAPAdrBlock;	/* LAPAdrBlock */
#define	dstNodeID	dst
#define	srcNodeID	src
#define	lapProtType	type

/* LAP definitions */
#define	lapDDPS		1	/* short DDP type */
#define	lapShortDDP	1	/* short DDP type */
#define	lapDDP		2	/* DDP type */
#define	lapSize		3	/* size of lap header */


struct DDP {			/* DDP */
	u_short	length;
	u_short	checksum;
	u_short	dstNet;
	u_short	srcNet;
	u_char	dstNode;
	u_char	srcNode;
	u_char	dstSkt;
	u_char	srcSkt;
	u_char	type;
};

struct ShortDDP {		/* ShortDDP */
	u_short	length;
	u_char	dstSkt;
	u_char	srcSkt;
	u_char	type;
};

/* DDP definitions */
#define	ddpMaxWKS	0x7F
#define	ddpMaxData	586
#define	ddpLengthMask	0x3FF
#define	ddpHopShift	10
#define	ddpSize		13	/* size of DDP header */
#define	ddpSSize	5
#define	ddpWKS		128	/* boundary of DDP well known sockets */
#define	ddpRTMP		1	/* RTMP type */
#define	ddpRTMPR	5	/* RTMP 'request' */
#define	ddpNBP		2	/* NBP type */
#define	ddpATP		3	/* ATP type */
#define	ddpECHO		4	/* ECHO type */
#define	ddpZIP		6	/* ZIP type */
#define	ddpIP		22	/* IP type */
#define	ddpARP		23	/* ARP type */

#define	echoSkt		4	/* ddp ECHO socket */
#define	echoReq		1	/* request echo */
#define	echoReply	2	/* reply to echo request */

#define	defddpWKSUnix	768	/* start of default (old) WKS range on UNIX */
#define	ddpNWKSUnix	16384	/* start of non-WKS .. */
#define	ddpIPSkt	72	/* socket used by Dartmouth encapsulation */

#define	ddp2ipskt(dskt) (((dskt) & ddpWKS) ? (dskt) + ddpNWKSUnix \
	: (dskt) + ddpWKSUnix)
#define ip2ddpskt(iskt) (((iskt) > ddpNWKSUnix) ? (iskt) - ddpNWKSUnix \
			 : (iskt) - ddpWKSUnix)

struct RTMP {			/* RTMP */
	u_short	net;
	u_char	idLen;
	u_char	id;		/* start of ID field */
};

struct RTMPtuple {
	u_short	net;
	u_char	hops;
};
#define	rtmpSkt	1		/* number of RTMP socket */
#define	rtmpSize	4	/* minimum size */
#define	rtmpTupleSize	3


struct ZIP {			/* ZIP */
	u_char	command;
	u_char	count;
};
#define	zipSkt		6	/* number of ZIP socket */
#define	zipQuery	1
#define	zipReply	2
#define	zipTakedown	3
#define	zipBringup	4
#define	zipSize		2

#define	zipGMZ		7	/* Get My Zone */
#define	zipGZL		8	/* Get Zone List */


typedef struct {		/* NBPTuple */
	AddrBlock addr;
	u_char	enume;
	u_char	name[3];	/* minimum length */
} NBPTuple;


struct NBP {			/* NBP */
	u_char	control:4;
	u_char	tuples:4;
	u_char	id;
	NBPTuple tuple;
};

typedef struct {		/* Entity, unpacked */
	u_char	obj[34];
	u_char	type[34];
	u_char	zone[34];
} Entity;

#define	nbpBrRq		0x1
#define	nbpLkUp		0x2
#define	nbpLkUpReply	0x3
#define	nbpNIS		2
#define	nbpMinSize	7		/* sizeof (struct NBP) - 3 */
#define	nbpTupleMin	5		/* sizeof (NBPTuple) - 3 */
#define	nbpTupleMax	15
#define	nbpEquals	'='
#define	nbpStar		'*'
	

struct ATP {			/* ATP */
	u_char	control;
	u_char	bitmap;
	u_short	transID;
	long	userData;
};
#define	atpSize		8	/* sizeof struct ATP */
#define	atpReqCode	0x40
#define	atpRspCode	0x80
#define	atpRelCode	0xC0
#define	atpCodeMask	0xC0	/* code mask bits */
#define	atpXO		0x20
#define	atpEOM		0x10
#define	atpSTS		0x08
#define	atpSendChk	0x01
#define	atpTIDValid	0x02
#define	atpFlagMask	0x3F
#define	atpControlMask	0xF8
#define	atpMaxNum	8
#define	atpMaxData	578
#define	atpNData	512	/* normal amount of data */


typedef struct {		/* Str32 */
	u_char	s[32];
} Str32;

typedef struct {		/* EntityName */
	Str32	objStr;
	Str32	typeStr;
	Str32	zoneStr;
} EntityName;

typedef struct {		/* RetransType */
	u_char	retransInterval;
	u_char	retransCount;
} RetransType;
